@@ -1,15 +1,16 @@
 # -*- coding: UTF-8 -*-
-"""Provide the :class:`OSV` class which allow to access dynamically to all
-methods proposed by an OSV model of the `OpenERP` server."""
+"""Provide the :class:`Model` class which allow to access dynamically to all
+methods proposed by an model of the `OpenERP` server."""
 
 import collections
 
+from oerplib.tools import v
 from oerplib import error
 from oerplib.service.osv import fields, browse
 
 
-class OSV(collections.Mapping):
-    """.. versionadded:: 0.5.0
+class Model(object):
+    """.. versionadded:: 0.5
 
     Represent a data model from the `OpenERP` server.
 
@@ -20,39 +21,41 @@ class OSV(collections.Mapping):
     >>> import oerplib
     >>> oerp = oerplib.OERP('localhost')
     >>> user = oerp.login('admin', 'passwd', 'database')
-    >>> user_osv = oerp.get('res.users')
-    >>> user_osv
-    <oerplib.service.osv.osv.OSV object at 0xb75ba4ac>
-    >>> user_osv.name_get(user.id) # Use any methods from the OSV instance
+    >>> user_obj = oerp.get('res.users')
+    >>> user_obj
+    <oerplib.service.osv.osv.Model object at 0xb75ba4ac>
+    >>> user_obj.name_get(user.id) # Use any methods from the model instance
     [[1, 'Administrator']]
 
     .. warning::
 
         The only method implemented in this class is ``browse``. Except this
         one, method calls are purely dynamic. As long as you know the signature
-        of the OSV method targeted, you will be able to use it
+        of the model method targeted, you will be able to use it
         (see the :ref:`tutorial <tutorials-execute-queries>`).
 
     """
 
     fields_reserved = ['id', '__oerp__', '__osv__', '__data__']
-    def __init__(self, oerp, osv_name):
-        super(OSV, self).__init__()
+
+    def __init__(self, oerp, model):
+        super(Model, self).__init__()
         self._oerp = oerp
-        self._browse_class = self._generate_browse_class(osv_name)
+        self._name = model
+        self._browse_class = self._generate_browse_class()
 
     def _browse_generator(self, ids, context=None):
         """Generator used by the
-        :func:`browse <oerplib.OERP.service.osv.osv.OSV.browse>` method.
+        :func:`browse <oerplib.OERP.service.osv.Model.browse>` method.
 
         """
         for o_id in ids:
             yield self.browse(o_id, context)
 
     def browse(self, ids, context=None):
-        """Browse one record or several records (if ``ids`` is a list of IDs)
-        according to the model ``osv_name``. The fields and values for such
-        objects are generated dynamically.
+        """Browse one or several records (if `ids` is a list of IDs)
+        from `model`. The fields and values for such objects are generated
+        dynamically.
 
         >>> oerp.get('res.partner').browse(1)
         browse_record(res.partner, 1)
@@ -76,22 +79,21 @@ class OSV(collections.Mapping):
             return obj
             #return self.browse(ids, context)
 
-    def _generate_browse_class(self, osv_name):
+    def _generate_browse_class(self):
         """Generate a class with all its fields corresponding to
-        the OSV name supplied and return them.
+        the model name supplied and return them.
 
         """
         # Retrieve server fields info and generate corresponding local fields
-        fields_get = self._oerp.execute(osv_name, 'fields_get')
-        cls_name = osv_name.replace('.', '_')
+        fields_get = self._oerp.execute(self._name, 'fields_get')
+        cls_name = self._name.replace('.', '_')
         if type(cls_name) == unicode:
             cls_name = cls_name.encode('utf-8')
         cls_fields = {}
         for field_name, field_data in fields_get.items():
-            if field_name not in OSV.fields_reserved:
-                cls_fields[field_name] = fields.generate_field(self,
-                                                               field_name,
-                                                               field_data)
+            if field_name not in Model.fields_reserved:
+                cls_fields[field_name] = fields.generate_field(
+                    self, field_name, field_data)
         # Case where no field 'name' exists, we generate one (which will be
         # in readonly mode) in purpose to be filled with the 'name_get' method
         if 'name' not in cls_fields:
@@ -100,7 +102,7 @@ class OSV(collections.Mapping):
 
         cls = type(cls_name, (browse.BrowseRecord,), {})
         cls.__oerp__ = self._oerp
-        cls.__osv__ = {'name': osv_name, 'columns': cls_fields}
+        cls.__osv__ = {'name': self._name, 'columns': cls_fields}
         slots = ['__oerp__', '__osv__', '__dict__', '__data__']
         slots.extend(cls_fields.keys())
         cls.__slots__ = slots
@@ -113,7 +115,7 @@ class OSV(collections.Mapping):
         for field_name in obj_data['fields_updated']:
             if field_name in obj_data['raw_data']:
                 field = self._browse_class.__osv__['columns'][field_name]
-                field_value = getattr(obj, "_{0}".format(field_name))
+                field_value = obj.__data__['values'][field_name]
                 # Many2One fields
                 if isinstance(field, fields.Many2OneField):
                     vals[field_name] = field_value and field_value[0]
@@ -121,12 +123,16 @@ class OSV(collections.Mapping):
                 else:
                     vals[field_name] = field_value
         try:
-            res = self._oerp.write(obj.__osv__['name'], [obj.id], vals, context)
+            if v(self._oerp._version) < v('6.1'):
+                res = self.write([obj.id], vals, context)
+            else:
+                res = self.write([obj.id], vals, context=context)
         except error.Error as exc:
             raise exc
         else:
             # Update raw_data dictionary
-            self._refresh(obj, context) #FIXME delete to avoid a RPC request?
+            # FIXME: make it optional to avoid a RPC request?
+            self._refresh(obj, context)
             return res
 
     def _refresh(self, obj, context=None):
@@ -135,24 +141,46 @@ class OSV(collections.Mapping):
         in the purpose to cancel all changes made.
 
         """
+        context = context or self._oerp.context
         obj_data = obj.__data__
-        obj_data['raw_data'] = self._oerp.read(obj.__osv__['name'], obj.id)
-        if obj_data['raw_data'] is False:
-            raise error.RPCError(
-                u"There is no '{osv_name}' record with ID {obj_id}.".format(
-                    osv_name=obj.__class__.__osv__['name'], obj_id=obj.id))
-        # Special field 'name' have to be filled with the value returned
-        # by the 'name_get' method
-        #try:
-        #    name = self._oerp.execute(obj.__osv__['name'], 'name_get', [obj.id])
-        #except error.Error:
-        #    pass
-        #else:
-        #    if name:
-        #        try:
-        #            obj_data['raw_data']['name'] = ast.literal_eval(name[0][1])
-        #        except Exception:
-        #            obj_data['raw_data']['name'] = name[0][1]
+        obj_data['context'] = context
+        # Get basic fields (no relational ones)
+        basic_fields = []
+        for field_name, field in obj.__osv__['columns'].iteritems():
+            if not getattr(field, 'relation', False):
+                basic_fields.append(field_name)
+            else:
+                obj_data['raw_data'][field_name] = None
+        # Fill fields with values of the record
+        if obj.id:
+            if v(self._oerp._version) < v('6.1'):
+                data = self.read([obj.id], basic_fields, context)
+                if data:
+                    obj_data['raw_data'].update(data[0])
+                else:
+                    obj_data['raw_data'] = False
+            else:
+                data = self.read([obj.id], basic_fields, context=context)
+                if data:
+                    obj_data['raw_data'].update(data[0])
+                else:
+                    obj_data['raw_data'] = False
+            if obj_data['raw_data'] is False:
+                raise error.RPCError(
+                    u"There is no '{model}' record with ID {obj_id}.".format(
+                        model=obj.__class__.__osv__['name'], obj_id=obj.id))
+        # No ID: fields filled with default values
+        else:
+            if v(self._oerp._version) < v('6.1'):
+                default_get = self.default_get(
+                    obj.__osv__['columns'].keys(), context)
+            else:
+                default_get = self.default_get(
+                    obj.__osv__['columns'].keys(), context=context)
+            obj_data['raw_data'] = {}
+            for field_name in obj.__osv__['columns']:
+                obj_data['raw_data'][field_name] = False
+            obj_data['raw_data'].update(default_get)
         self._reset(obj)
 
     def _reset(self, obj):
@@ -166,22 +194,41 @@ class OSV(collections.Mapping):
         # Load fields and their values
         for field in self._browse_class.__osv__['columns'].values():
             if field.name in obj_data['raw_data']:
-                setattr(obj, "_{0}".format(field.name),
-                        obj_data['raw_data'][field.name])
+                obj_data['values'][field.name] = \
+                    obj_data['raw_data'][field.name]
                 setattr(obj.__class__, field.name,
                         field)
 
     def _unlink_record(self, obj, context=None):
         """Delete the object from the OpenERP server."""
-        return self._oerp.unlink(obj.__osv__, [obj.id], context)
+        if v(self._oerp._version) < v('6.1'):
+            return self.unlink([obj.id], context)
+        else:
+            return self.unlink([obj.id], context=context)
 
     def __getattr__(self, method):
-        def rpc_method(*args):
-            result = self._oerp.execute(self._browse_class.__osv__['name'],
-                                        method, *args)
+        """Provide a dynamic access to a RPC method."""
+        def rpc_method(*args, **kwargs):
+            """Return the result of the RPC request."""
+            if v(self._oerp._version) < v('6.1'):
+                if kwargs:
+                    raise error.RPCError(
+                        u"Named parameters are not supported by this version "
+                        u"of OpenERP")
+                result = self._oerp.execute(
+                    self._browse_class.__osv__['name'], method, *args)
+            else:
+                if self._oerp.config['auto_context'] \
+                        and 'context' not in kwargs:
+                    kwargs['context'] = self._oerp.context
+                result = self._oerp.execute_kw(
+                    self._browse_class.__osv__['name'], method, args, kwargs)
             return result
         return rpc_method
 
+    def __repr__(self):
+        return "Model(%r)" % (self._browse_class.__osv__['name'])
+
     # ---------------------------- #
     # -- MutableMapping methods -- #
     # ---------------------------- #
@@ -190,9 +237,8 @@ class OSV(collections.Mapping):
         return self.browse(obj_id)
 
     def __iter__(self):
-        ids = self._oerp.search(self._browse_class.__osv__['name'])
-        for obj_id in ids:
-            yield self.browse(obj_id)
+        ids = self.search([])
+        return self._browse_generator(ids)
 
     def __len__(self):
         return self._oerp.search(self._browse_class.__osv__['name'], count=True)
